' Copyright  2010 Corel Corporation.
' Originally made available on http://CorelDRAW.com/blogs/insider
'
' Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the software and
' accompanying documentation covered by this license (the "Software") to use, reproduce, display, distribute, execute,
' and transmit the Software, and to prepare derivative works of the Software, and to permit third-parties to whom the
' Software is furnished to do so, all subject to the following:
'
' The copyright notices in the Software and this entire statement, including the above license grant, the original
' location it was downloaded from, this restriction and the following disclaimer, must be included in all copies of
' the Software, in whole or in part, and all derivative works of the Software, unless such copies or derivative works
' are solely in the form of machine-executable object code generated by a source language processor.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
' WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. THE SOFTWARE MAY CONTAIN
' BUGS, ERRORS AND OTHER PROBLEMS THAT COULD CAUSE SYSTEM FAILURES AND THE USE OF SUCH SOFTWARE IS ENTIRELY AT THE
' USERS RISK. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR
' OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
' THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'
'
' Note:
'  This is the VBA side of the table preset docker.  It demonstrates how to couple vba and javascript to
'  save/load presets and generate javascript and html code.
'                                                                                        Hendrik, 2010

Option Explicit On

Private Declare Function SetFocus Lib "user32" (ByVal hwnd As Long) As Long

Sub ShowTablePresetDocker()
    Application.FrameWork.ShowDocker("25a4ac41-cdfd-738e-4daf-39e1f25763ca")
End Sub

Function CreateList() As String
    Dim presetFolder As String
    Dim folder As String
    ' get the table presets addon folder
    presetFolder = Application.SetupPath & "Programs\Addons\TablePresetDocker\"

    CreateList = CreateList & "<br/>"
    ' iterate through the folders
    folder = Dir(presetFolder & "*.*", vbDirectory)
    While folder <> ""
        If folder <> "." And folder <> ".." And (GetAttr(presetFolder & folder) And vbDirectory) <> 0 Then
            ' add the presets to the listbox (actually to the webpage)
            CreateList = CreateList & "<div align=center id=""" & folder & """ OnMouseEnter=""OnEnter('" & folder & "')"""
            CreateList = CreateList & "OnMouseLeave=""OnLeave('" & folder & "')"" OnClick=""Click('" & folder & "')"">"
            CreateList = CreateList & "<img src=""" & folder & "\preview.png""/><br/>"
            CreateList = CreateList & folder
            CreateList = CreateList & "</div><br/>"
        End If
        folder = Dir()  ' get the next folder
    End While
End Function

Function IsTableSelected() As Boolean
    IsTableSelected = Not ActiveSelection.Shapes.FindShape(Query:="@type='plugin:table'") Is Nothing
End Function

Function GetSelectedTables() As ShapeRange
    GetSelectedTables = ActiveSelection.Shapes.FindShapes(Query:="@type='plugin:table' and (@com.ParentGroup = null or @com.ParentGroup.Type <> 21)")
End Function

Function ReadPreset(ByVal preset As String) As String
    On Error GoTo errorHandler
    Dim presetFile As String
    Dim buff() As Byte
    ' get the table preset filename
    presetFile = Application.SetupPath & "Programs\Addons\TablePresetDocker\" & preset & "\preset.js"
    ' open the file
    Open presetFile For Binary Access Read As #1
    ' size the buffer to the file size
    ReDim buff(0 To LOF(1) - 1)
    Get #1, , buff
    Close #1
    ' convert the string to unicode
    ReadPreset = StrConv(buff, vbUnicode)

    Exit Function
errorHandler:
    Close #1
    ReadPreset = ""
End Function

Sub SolidColor(ByVal table As Shape, ByVal usec As Boolean, ByVal c As Color)
    table.Fill.ApplyNoFill()
    If usec Then
        table.Custom.Cells.All.ApplyFill(ActiveDocument.CreateUniformFill(c))
    Else
        table.Custom.Cells.All.ApplyFill(ActiveDocument.CreateFill)
    End If
End Sub

Sub AlternatingRows(ByVal table As Shape, ByVal usec1 As Boolean, ByVal c1 As Color, _
                    ByVal usec2 As Boolean, ByVal c2 As Color)
    Dim i As Integer
    table.Fill.ApplyNoFill()
    For i = 1 To table.Custom.Rows.Count
        If i And 1 Then
            If usec1 Then
                table.Custom.Rows(i).Cells.All.ApplyFill(ActiveDocument.CreateUniformFill(c1))
            Else
                table.Custom.Rows(i).Cells.All.ApplyFill(ActiveDocument.CreateFill)
            End If
        Else
            If usec2 Then
                table.Custom.Rows(i).Cells.All.ApplyFill(ActiveDocument.CreateUniformFill(c2))
            Else
                table.Custom.Rows(i).Cells.All.ApplyFill(ActiveDocument.CreateFill)
            End If
        End If
    Next i
End Sub

Sub AlternatingCols(ByVal table As Shape, ByVal usec1 As Boolean, ByVal c1 As Color, _
                    ByVal usec2 As Boolean, ByVal c2 As Color)
    Dim i As Integer
    table.Fill.ApplyNoFill()
    For i = 1 To table.Custom.Columns.Count
        If i And 1 Then
            If usec1 Then
                table.Custom.Columns(i).Cells.All.ApplyFill(ActiveDocument.CreateUniformFill(c1))
            Else
                table.Custom.Columns(i).Cells.All.ApplyFill(ActiveDocument.CreateFill)
            End If
        Else
            If usec2 Then
                table.Custom.Columns(i).Cells.All.ApplyFill(ActiveDocument.CreateUniformFill(c2))
            Else
                table.Custom.Columns(i).Cells.All.ApplyFill(ActiveDocument.CreateFill)
            End If
        End If
    Next i
End Sub

Sub Set3dOutlines(ByVal table As Shape, _
                  ByVal usec1 As Boolean, ByVal width1 As Double, ByVal c1 As Color, _
                  ByVal usec2 As Boolean, ByVal width2 As Double, ByVal c2 As Color, _
                  ByVal usefc As Boolean, ByVal fc As Color, _
                  ByVal showOutside As Boolean, ByVal spacing As Double)
    Dim out As Outline
    out = ActiveDocument.CreateOutline

    ActiveDocument.SaveSettings()
    ActiveDocument.Unit = cdrMillimeter

    table.Custom.SeparatedBorders = True
    table.Custom.SetCellSpacing(spacing, spacing)

    If usec1 Then
        out.SetProperties(width1, , c1)
    Else
        out.SetNoOutline()
    End If

    table.Custom.Cells.All.Borders.SetBorders(3, out)
    If showOutside Then
        table.Custom.Borders.SetBorders(12, out)
    End If

    If usec2 Then
        out.SetProperties(width2, , c2)
    Else
        out.SetNoOutline()
    End If

    table.Custom.Cells.All.Borders.SetBorders(12, out)
    If showOutside Then
        table.Custom.Borders.SetBorders(3, out)
    End If

    If Not showOutside Then
        out.SetNoOutline()
        table.Custom.Borders.SetBorders(15, out)
    End If

    If usefc Then
        table.Fill.ApplyUniformFill(fc)
    Else
        table.Fill.ApplyNoFill()
    End If

    ActiveDocument.RestoreSettings()

End Sub

Sub SetOutlines(ByVal table As Shape, _
                ByVal usec1 As Boolean, ByVal width1 As Double, ByVal c1 As Color, _
                ByVal usec2 As Boolean, ByVal width2 As Double, ByVal c2 As Color)
    Dim out As Outline
    out = ActiveDocument.CreateOutline

    ActiveDocument.SaveSettings()
    ActiveDocument.Unit = cdrMillimeter

    If usec2 Then
        out.SetProperties(width2, , c2)
    Else
        out.SetNoOutline()
    End If

    table.Custom.SeparatedBorders = False
    table.Custom.Borders.SetBorders(15, out)

    If usec1 Then
        out.SetProperties(width1, , c1)
    Else
        out.SetNoOutline()
    End If

    table.Custom.Cells.All.Borders.SetBorders(48, out)

    ActiveDocument.RestoreSettings()
End Sub

Sub SetFocusToDoc()
    SetFocus(ActiveWindow.Handle)
End Sub

Function BtoS(ByVal b As Boolean) As String
    BtoS = LCase(CStr(b))
End Function

Sub ApplySettings(ByVal ftype As Integer, _
                  ByVal usefc1 As Boolean, ByVal fc1 As Color, _
                  ByVal usefc2 As Boolean, ByVal fc2 As Color, _
                  ByVal otype As Integer, _
                  ByVal useoc1 As Boolean, ByVal w1 As Double, ByVal oc1 As Color, _
                  ByVal useoc2 As Boolean, ByVal w2 As Double, ByVal oc2 As Color, _
                  ByVal usefc As Boolean, ByVal fc As Color, _
                  ByVal spacing As Double, ByVal showOutside As Boolean)
    Dim sr As ShapeRange
    Dim s As Shape
    Dim i As Integer

    ActiveDocument.SaveSettings()
    ActiveDocument.Unit = cdrMillimeter

    sr = GetSelectedTables
    ActiveDocument.BeginCommandGroup("Apply table settings")
    For Each s In sr
        If ftype = 0 Then   ' solid
            SolidColor(s, usefc1, fc1)
        ElseIf ftype = 1 Then ' alternating rows
            AlternatingRows(s, usefc1, fc1, usefc2, fc2)
        ElseIf ftype = 2 Then ' alternating columns
            AlternatingCols(s, usefc1, fc1, usefc2, fc2)
        End If

        If otype = 0 Then ' 2d outlines
            SetOutlines(s, useoc1, w1, oc1, useoc2, w2, oc2)
        Else ' 3d outlines
            Set3dOutlines(s, useoc1, w1, oc1, useoc2, w2, oc2, usefc, fc, showOutside, spacing)
        End If

    Next s
    ActiveDocument.EndCommandGroup()
    ActiveDocument.RestoreSettings()
End Sub

Function SavePreset(ByVal presetname As String, _
                    ByVal ftype As Integer, _
                    ByVal usefc1 As Boolean, ByVal fc1 As Color, _
                    ByVal usefc2 As Boolean, ByVal fc2 As Color, _
                    ByVal otype As Integer, _
                    ByVal useoc1 As Boolean, ByVal w1 As Double, ByVal oc1 As Color, _
                    ByVal useoc2 As Boolean, ByVal w2 As Double, ByVal oc2 As Color, _
                    ByVal usefc As Boolean, ByVal fc As Color, _
                    ByVal spacing As Double, ByVal showOutside As Boolean) As Boolean
    SavePreset = False

    On Error GoTo errorHandler
    Dim presetFile As String
    Dim previewFile As String
    Dim presetFolder As String
    Dim javascript As String
    Dim backupSel As ShapeRange
    Dim filter As ExportFilter

    backupSel = ActiveSelectionRange
    ' get the table preset filename
    presetFolder = Application.SetupPath & "Programs\Addons\TablePresetDocker\" & presetname
    presetFile = presetFolder & "\preset.js"
    previewFile = presetFolder & "\preview.png"

    ActiveDocument.SaveSettings()
    ActiveDocument.Unit = cdrMillimeter

    ActiveDocument.BeginCommandGroup("Save Preset")
    Optimization = True

    ' create a table to apply the settings to.  This table will be the preview that we export to png
    Dim s1 As Shape
    s1 = ActiveLayer.CreateCustomShape("Table", 0, 0, 100, 100, 4, 4)

    If Dir(presetFolder, vbDirectory) = "" Then
        MkDir(presetFolder)
    End If

    ' see if the preset already exists
    If Dir(presetFile) <> "" Then
        If MsgBox("Preset name exists.  Would you like to replace preset?", vbYesNo, "Warning") = vbNo Then
            GoTo done
        End If
    End If

    ' open the preset file to write javascript code to
        Open presetFile For Output As #1

    If ftype = 0 Then   ' solid
        javascript = "app.GMSManager.RunMacro(""TablePresetDocker"", ""Macros.SolidColor"", table, " & BtoS(usefc1) & ", app.CreateColor("""
        javascript = javascript & fc1.ToString & """));"
        SolidColor(s1, usefc1, fc1)
    ElseIf ftype = 1 Then ' alternating rows
        javascript = "app.GMSManager.RunMacro(""TablePresetDocker"", ""Macros.AlternatingRows"", table, " & BtoS(usefc1) & ", app.CreateColor("""
        javascript = javascript & fc1.ToString & """), " & BtoS(usefc2) & ", app.CreateColor(""" & fc2.ToString & """));"
        AlternatingRows(s1, usefc1, fc1, usefc2, fc2)
    ElseIf ftype = 2 Then ' alternating columns
        javascript = "app.GMSManager.RunMacro(""TablePresetDocker"", ""Macros.AlternatingCols"", table, " & BtoS(usefc1) & ", app.CreateColor("""
        javascript = javascript & fc1.ToString & """), " & BtoS(usefc2) & ", app.CreateColor(""" & fc2.ToString & """));"
        AlternatingCols(s1, usefc1, fc1, usefc2, fc2)
    End If

        Print #1, javascript

    If otype = 0 Then ' 2d outlines
        javascript = "app.GMSManager.RunMacro(""TablePresetDocker"", ""Macros.SetOutlines"", table, "
        javascript = javascript & BtoS(useoc1) & ", " & CStr(w1) & ", app.CreateColor(""" & oc1.ToString & """),"
        javascript = javascript & BtoS(useoc2) & ", " & CStr(w2) & ", app.CreateColor(""" & oc2.ToString & """));"
        SetOutlines(s1, useoc1, w1, oc1, useoc2, w2, oc2)
    Else ' 3d outlines
        javascript = "app.GMSManager.RunMacro(""TablePresetDocker"", ""Macros.Set3dOutlines"", table, "
        javascript = javascript & BtoS(useoc1) & ", " & CStr(w1) & ", app.CreateColor(""" & oc1.ToString & """),"
        javascript = javascript & BtoS(useoc2) & ", " & CStr(w2) & ", app.CreateColor(""" & oc2.ToString & """),"
        javascript = javascript & BtoS(usefc) & ", app.CreateColor(""" & fc.ToString & """), " & BtoS(showOutside) & ", " & CStr(spacing) & ");"
        Set3dOutlines(s1, useoc1, w1, oc1, useoc2, w2, oc2, usefc, fc, showOutside, spacing)
    End If

        Print #1, javascript

        Close #1

    ' export the png file for the table preset preview
    filter = ActiveDocument.ExportBitmap(previewFile, cdrPNG, cdrSelection, cdrRGBColorImage, 80, 80)
    filter.Finish()

done:
    ActiveDocument.EndCommandGroup()
    ActiveDocument.Undo()                ' undo to remove the table preset preview
    Optimization = False
    backupSel.CreateSelection()
    ActiveDocument.RestoreSettings()

    SavePreset = True
    Exit Function
errorHandler:
    Close #1
    Resume done
End Function


